/*  GNU OCRAD - Optical Character Recognition program
    Copyright (C) 2003 Antonio Diaz Diaz.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

class Blockmap;

class Block : public Rectangle
  {
  std::list< Block > _block_list; // list of blocks nested into this one
  const Blockmap * _blockmap;	// Blockmap to which this block belongs
  int _id;			// block identification number
				// positive for block of black dots
				// negative for block of white dots
public:
  Block( const Rectangle & r, const Blockmap & b, int id ) throw()
    : Rectangle( r ), _blockmap( &b ), _id( id ) {}

  const std::list< Block > & block_list() const throw() { return _block_list; }
  const Blockmap * blockmap() const throw() { return _blockmap; }

  int area() const throw();
  int id() const throw() { return _id; }
  int id( int row, int col ) const throw();
  int blocks() const throw() { return _block_list.size(); }
  bool compare_id( int id, bool recursive ) const throw();

  bool escape_left  ( int row, int col ) const throw();
  bool escape_top   ( int row, int col ) const throw();
  bool escape_right ( int row, int col ) const throw();
  bool escape_bottom( int row, int col ) const throw();
  int follow_top   ( int row, int col ) const throw();
  int follow_bottom( int row, int col ) const throw();
//  bool seek_around( int *row, int *col, bool black = true ) const throw();
  int seek_left  ( int row, int col, bool black = true ) const throw();
  int seek_top   ( int row, int col, bool black = true ) const throw();
  int seek_right ( int row, int col, bool black = true ) const throw();
  int seek_bottom( int row, int col, bool black = true ) const throw();
  bool top_hook   ( int *hdiff ) const throw();
  bool bottom_hook( int *hdiff ) const throw();

  void print( FILE * outfile, int sp ) const throw();

  friend void hierarchize_blocks( std::list< Block > & block_list ) throw();
  };
